package kr.co.mkblog.mkblogopenglesexample;

import android.content.Context;
import android.opengl.GLES30;
import android.util.Log;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.IntBuffer;

/**
 * Created by mkblog on 2018-10-29
 */
public class mkGLExample {
    //MK: Triangle Vertex
    private float[] mkVertices = {  0.5f, 0.5f, 0.0f, 1.0f, 0.0f, 0.0f, 0.55f, 0.55f, //MK: Top Right
            0.5f, -0.5f, 0.0f, 0.0f, 1.0f, 0.0f, 0.55f, 0.45f,//MK: Bottom Right
            -0.5f, -0.5f, 0.0f, 1.0f, 0.0f, 0.0f, 0.45f, 0.45f, //MK: Bottom Left
            -0.5f, 0.5f, 0.0f, 0.0f, 0.0f, 1.0f, 0.45f, 0.55f //MK: Top Left
    };

    private int[] mkIndices = { 0, 1, 3, //MK: First Triangle
            1, 2, 3 //MK: Second Triangle
    };

    private int[] mkVAO;
    private int[] mkVBO;
    private int[] mkEBO;

    private mkShader mkTriShader;
    private mkTextureLoader brickTexture;
    private mkTextureLoader wallTexture;
    private mkTextureLoader smileTexture;

    public mkGLExample(Context inputContext){

        mkTriShader = new mkShader(inputContext, "mkVertexCode", "mkFragmentCode");

        //MK: Make VAO, VBO
        mkVAO = new int[1];
        GLES30.glGenVertexArrays(1, mkVAO, 0);

        mkVBO = new int[1];
        GLES30.glGenBuffers(1, mkVBO, 0);

        //MK: Make Element Buffer Object (EBO)
        mkEBO = new int[1];
        GLES30.glGenBuffers(1, mkEBO, 0);

        //MK: VAO Bind 이후에 Bind되는 VBO, EBO는 VAO에 저장되는 것으로 판단됨
        GLES30.glBindVertexArray(mkVAO[0]);

        //MK: VBO Bind
        GLES30.glBindBuffer(GLES30.GL_ARRAY_BUFFER, mkVBO[0]);

        //MK: Vertex 좌표를 mkVBO로 복사하는 과정
        ByteBuffer mkByteBuffer = ByteBuffer.allocateDirect(mkVertices.length * 4);
        mkByteBuffer.order(ByteOrder.nativeOrder());
        FloatBuffer mkVertexBuffer = mkByteBuffer.asFloatBuffer();
        mkVertexBuffer.put(mkVertices);
        mkVertexBuffer.position(0);

        GLES30.glBufferData(GLES30.GL_ARRAY_BUFFER, mkVertices.length*4, mkVertexBuffer, GLES30.GL_STATIC_DRAW );

        //MK: mkVBO의 Offset, Vec사이즈 등을 설정하는 부분
        GLES30.glVertexAttribPointer(0, 3, GLES30.GL_FLOAT, false, 8 * 4, 0);
        //MK: mkVBO가 한개 Bind되어서 0으로 설정한듯 함 (계속 공부를 하면서 알아볼 필요가 있음)
        GLES30.glEnableVertexAttribArray(0);

        //MK: mkVBO의 Offset, Vec사이즈 등을 설정하는 부분
        GLES30.glVertexAttribPointer(1, 3, GLES30.GL_FLOAT, false, 8 * 4, 3 * 4);
        //MK: mkVBO가 한개 Bind되어서 0으로 설정한듯 함 (계속 공부를 하면서 알아볼 필요가 있음)
        GLES30.glEnableVertexAttribArray(1);

        //MK: Texture Coordinate 좌표를 받기 위해서 설정
        GLES30.glVertexAttribPointer(2, 2, GLES30.GL_FLOAT, false, 8 * 4, 6 * 4);
        GLES30.glEnableVertexAttribArray(2);

        //MK: EBO Bind
        GLES30.glBindBuffer(GLES30.GL_ELEMENT_ARRAY_BUFFER, mkEBO[0]);

        //MK: Indice 값을 mkEBO로 복사하는 과정
        IntBuffer mkIndexBuffer = mkByteBuffer.asIntBuffer();
        mkIndexBuffer.put(mkIndices);
        mkIndexBuffer.position(0);

        GLES30.glBufferData(GLES30.GL_ELEMENT_ARRAY_BUFFER, mkIndices.length*4, mkIndexBuffer, GLES30.GL_STATIC_DRAW);

        brickTexture = new mkTextureLoader(inputContext, "container", GLES30.GL_CLAMP_TO_EDGE, GLES30.GL_CLAMP_TO_EDGE);
        wallTexture = new mkTextureLoader(inputContext, "wall", GLES30.GL_CLAMP_TO_EDGE, GLES30.GL_CLAMP_TO_EDGE);
        smileTexture = new mkTextureLoader(inputContext, "awesomeface", GLES30.GL_REPEAT, GLES30.GL_REPEAT);

        mkTriShader.use(); //MK: Texture가 1개일 때는 정상적으로 동작하는 것 같음. 하지만 2개가 되는 순간부터 마지막만 로딩되는 현상이 있음.
        mkTriShader.setUniformInt("inputTexture", 0);
        mkTriShader.setUniformInt("inputTexture2", 1);
        mkTriShader.setUniformInt("inputTexture3", 2);
    }

    public void draw(){
        //MK: 바로 위에 코드에 setUniformInt에서 설정한 뒤의 숫자에 따라 GL_TEXTURE0, 1등을 설정하는 것으로 판단됨
        GLES30.glActiveTexture(GLES30.GL_TEXTURE1);
        GLES30.glBindTexture(GLES30.GL_TEXTURE_2D, brickTexture.textureID[0]);
        GLES30.glActiveTexture(GLES30.GL_TEXTURE0);
        GLES30.glBindTexture(GLES30.GL_TEXTURE_2D, wallTexture.textureID[0]);
        GLES30.glActiveTexture(GLES30.GL_TEXTURE2);
        GLES30.glBindTexture(GLES30.GL_TEXTURE_2D, smileTexture.textureID[0]);

        //MK: Draw Triangles
        mkTriShader.use();
        GLES30.glBindVertexArray(mkVAO[0]);
        GLES30.glDrawElements(GLES30.GL_TRIANGLES, 6, GLES30.GL_UNSIGNED_INT, 0);
    }
}




